<?php

class OAuthSignature {

    //OAuth Parameter Names
    public $PARAM_OAUTH_CONSUMER_KEY = "oauth_consumer_key";
    public $PARAM_OAUTH_SIGNATURE = "oauth_signature";
    public $PARAM_OAUTH_APP_ID = "oauth_application_id";
    public $PARAM_OAUTH_SIGNATURE_METHOD = "oauth_signature_method";
    public $PARAM_OAUTH_TIMSTAMP = "oauth_timestamp";
    public $PARAM_OAUTH_NONCE = "oauth_nonce";
    public $PARAM_AUTH_NAME = "x-authorization";
    //Common Keys used both consumer & provider
    public $APP_ID = "MYAPP";
    public $CONSUMER_KEY = "MYCONSUMERKEY";
    public $SECRET_KEY = "SECRETKEY";
    public $SIGNATURE_METHOD = "HmacSHA1";
    public $CC_SHA1_DIGEST_LENGTH = 20;
    public $CC_SHA1_BLOCK_BYTES = 64;

    public function generateConsumerSignature() {

        //    return @"x-authorization=oauth_timestamp=\"1375528841\",oauth_nonce=\"3gVsBXKlsTsQQmWs74szIIPvHwIr3GSF\",oauth_signature=\"1JPyBnPLaGOQWLsxq6DCff97a24%3D\"";
        $oauthParams = array(
            $this->PARAM_OAUTH_CONSUMER_KEY => $this->CONSUMER_KEY,
            $this->PARAM_OAUTH_APP_ID => $this->APP_ID,
            $this->PARAM_OAUTH_SIGNATURE_METHOD => $this->SIGNATURE_METHOD,
            $this->PARAM_OAUTH_TIMSTAMP => $this->getTimestamp(),
            $this->PARAM_OAUTH_NONCE => $this->generateRandomString()
        );

        //echo "<pre>";
        //print_r($oauthParams);
        //return;

        $signature = $this->generateSignature($oauthParams);
        //print_r($oauthParams);

        $oauthParams[$this->PARAM_OAUTH_SIGNATURE] = $signature;

        //echo "signater: " . $signature . "<br/>";        
        //print_r($oauthParams);
        $sb = '';
        $i = 0;
        foreach ($oauthParams as $key => $val) {
            //No Need to send all the parameters as plain text over the URl as parameter. 
            // Just send timestamp, nonce & the signature            
            if ($key == $this->PARAM_OAUTH_TIMSTAMP || $key == $this->PARAM_OAUTH_NONCE || $key == $this->PARAM_OAUTH_SIGNATURE) {
                $i++;
                $sb.= $key . "=\"" . rawurlencode($val) . "\"";
                if (3 > $i) {
                    $sb.=",";
                }
            }
        }
        $xauth = $this->PARAM_AUTH_NAME . "=" . $sb;
        return $xauth;
    }

    public function getTimestamp() {
        //return "1381576477";
        $curdate = time();//strtotime(date('Y-m-d H:i:s'));
        return $curdate;
    }

    public function generateRandomString($length = 32) {
        //return "69jep9z8F1ZGjEht8mU7EtjwNxFirvso";
        $characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXZY0123456789';
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, strlen($characters) - 1)];
        }
        return $randomString;
    }

    public function generateSignature($oauthParams) {
        $encodedParams = $this->normalizeParams($oauthParams);
        //echo "generateSignature - encodedParams " . $encodedParams . "<br/>";

        return $this->generateHmacSHA1Signature($encodedParams);
    }

    public function normalizeParams($oauthParams) {
        $stringParams = "";
        ksort($oauthParams);

        //echo end($oauthParams);
        foreach ($oauthParams as $key => $value) {
            $stringParams.=$key . '=' . $value;
            if ($value != end($oauthParams)) {
                $stringParams.="&";
            }
        }

        //echo "normalizeParams - stringParams " . $stringParams . "<br/>";
        // URL-encode the equals ("%3D") and ampersand ("%26")
        $encodedParams = rawurlencode($stringParams);

        return $encodedParams;
    }

    public function generateHmacSHA1Signature($encodedParams) {

        //echo "Before HmacSHA1 : " . $encodedParams . "<br/>";
        
        $key = utf8_encode($this->SECRET_KEY);
        $encodedParams = utf8_encode($encodedParams);
        
        $digestLength = $this->CC_SHA1_DIGEST_LENGTH;
        
        //echo "digestLength : " . $digestLength . "<br/>";
        
        $CCHmac = hash_hmac("sha1", $encodedParams, $key, TRUE);
        
        $HMAC = substr($CCHmac, 0, $digestLength);        
        
        //echo "After HmacSHA1 : " . $hmacFormated . "<br/>";*/

        // Base64-encode the signature
        $hash = base64_encode($HMAC);
        //echo "Base64 Encoded : " . $hash . "<br/>";

        return $hash;
    }   

}

?>